""" Test cases for PtyProcess.wait method. """
import time
import unittest
from ptyprocess import PtyProcess


class TestWaitAfterTermination(unittest.TestCase):
    """Various test cases for PtyProcess.wait()"""

    def test_wait_true_shortproc(self):
        """Ensure correct (True) wait status for short-lived processes."""
        child = PtyProcess.spawn(['true'])
        # Wait so we're reasonable sure /bin/true has terminated
        time.sleep(0.2)
        self.assertEqual(child.wait(), 0)

    def test_wait_false_shortproc(self):
        """Ensure correct (False) wait status for short-lived processes."""
        child = PtyProcess.spawn(['false'])
        # Wait so we're reasonable sure /bin/false has terminated
        time.sleep(0.2)
        self.assertNotEqual(child.wait(), 0)

    def test_wait_twice_longproc(self):
        """Ensure correct wait status when called twice."""
        # previous versions of ptyprocess raises PtyProcessError when
        # wait was called more than once with "Cannot wait for dead child
        # process.".  No longer true since v0.5.
        child = PtyProcess.spawn(['sleep', '1'])
        # this call to wait() will block for 1s
        for count in range(2):
            self.assertEqual(child.wait(), 0, count)



#!/usr/bin/env python
'''
PEXPECT LICENSE

    This license is approved by the OSI and FSF as GPL-compatible.
        http://opensource.org/licenses/isc-license.txt

    Copyright (c) 2012, Noah Spurrier <noah@noah.org>
    PERMISSION TO USE, COPY, MODIFY, AND/OR DISTRIBUTE THIS SOFTWARE FOR ANY
    PURPOSE WITH OR WITHOUT FEE IS HEREBY GRANTED, PROVIDED THAT THE ABOVE
    COPYRIGHT NOTICE AND THIS PERMISSION NOTICE APPEAR IN ALL COPIES.
    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
    WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
    MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
    ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
    WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
    ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
    OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

'''
import time
import unittest
from ptyprocess import PtyProcess, PtyProcessUnicode
import errno
import os
import stat
import tempfile

class InvalidBinaryChars(unittest.TestCase):

    def test_invalid_binary(self):
        '''This tests that we correctly handle the case where we attempt to
           spawn a child process but the exec call fails'''

        # Create a file that should fail the exec call
        dirpath = tempfile.mkdtemp()
        fullpath = os.path.join(dirpath, "test")

        with open(fullpath, 'wb') as f:
            # Add some constant so it will never be executable
            #  - Not 0x54AD (Windows PE)
            #  - Not 0x7FEF (ELF)
            #  - Not 0410 or 0413 (a.out)
            #  - Not 0x2321 (script)
            file_start = b'\x00\x00'
            file_data = file_start + os.urandom(1022)
            f.write(file_data)

        # Make it executable
        st = os.stat(fullpath)
        os.chmod(fullpath, st.st_mode | stat.S_IEXEC)

        # TODO Verify this does what is intended on Windows
        try:
            child = PtyProcess.spawn([fullpath])
            # If we get here then an OSError was not raised
            child.close()
            raise AssertionError("OSError was not raised")
        except OSError as err:
            if errno.ENOEXEC == err.errno:
                # This is what should happen
                pass
            else:
                # Re-raise the original error to fail the test
                raise
        finally:
            os.unlink(fullpath)
            os.rmdir(dirpath)

if __name__ == '__main__':
    unittest.main()

suite = unittest.makeSuite(InvalidBinaryChars,'test')




import fcntl
import os
import time
import select
import tempfile
import unittest
from ptyprocess.ptyprocess import which
from ptyprocess import PtyProcess, PtyProcessUnicode

class PtyTestCase(unittest.TestCase):
    def setUp(self):
        self.cmd = u'echo $ENV_KEY; exit 0\n'
        self.env = os.environ.copy()
        self.env_key = u'ENV_KEY'
        self.env_value = u'env_value'
        self.env[self.env_key] = self.env_value

    def _canread(self, fd, timeout=1):
        return fd in select.select([fd], [], [], timeout)[0]

    def _spawn_sh(self, ptyp, cmd, outp, env_value):
        # given,
        p = ptyp.spawn(['sh'], env=self.env)
        p.write(cmd)

        # exercise,
        while True:
            try:
                outp += p.read()
            except EOFError:
                break

        # verify, input is echo to output
        assert cmd.strip() in outp

        # result of echo $ENV_KEY in output
        assert env_value in outp

        # exit successfully (exit 0)
        assert p.wait() == 0


    def test_spawn_sh(self):
        outp = b''
        self._spawn_sh(PtyProcess, self.cmd.encode('ascii'),
                       outp, self.env_value.encode('ascii'))

    def test_spawn_sh_unicode(self):
        outp = u''
        self._spawn_sh(PtyProcessUnicode, self.cmd,
                       outp, self.env_value)

    def test_quick_spawn(self):
        """Spawn a very short-lived process."""
        # so far only reproducible on Solaris 11, spawning a process
        # that exits very quickly raised an exception at 'inst.setwinsize',
        # because the pty file descriptor was quickly lost after exec().
        PtyProcess.spawn(['true'])

    def _interactive_repl_unicode(self, echo):
        """Test Call and response with echo ON/OFF."""
        # given,
        bc = PtyProcessUnicode.spawn(['bc'], echo=echo)
        given_input = u'2+2+2+2+2+2+2+2+2+2+2+2+2+2+2+2+2+2+2+2\n'
        expected_output = u'40'

        # gnu-bc will display a long FSF banner on startup,
        # whereas bsd-bc (on FreeBSD, Solaris) display no
        # banner at all.  To ensure we've read up to our
        # current prompt, read until the response of '2^16' is found.
        time.sleep(1)

        bc.write(u'2^16\n')
        outp = u''
        while self._canread(bc.fd):
            outp += bc.read()
        assert u'65536' in outp

        # exercise,
        bc.write(given_input)

        while self._canread(bc.fd, timeout=2):
            outp += bc.read()

        # with echo ON, we should see our input.
        #
        # note: we cannot assert the reverse: on Solaris, FreeBSD,
        # and OSX, our input is echoed to output even with echo=False,
        # something to do with the non-gnu version of bc(1), perhaps.
        if echo:
            assert given_input.strip() in outp

        # we should most certainly see the result output.
        assert expected_output in outp

        # exercise sending EOF
        bc.sendeof()

        # validate EOF on read
        while True:
            try:
                bc.read()
            except EOFError:
                break

        # validate exit status,
        assert bc.wait() == 0

    @unittest.skipIf(which('bc') is None, "bc(1) not found on this server.")
    def test_interactive_repl_unicode_noecho(self):
        self._interactive_repl_unicode(echo=False)

    @unittest.skipIf(which('bc') is None, "bc(1) not found on this server.")
    def test_interactive_repl_unicode_echo(self):
        self._interactive_repl_unicode(echo=True)

    def test_pass_fds(self):
        with tempfile.NamedTemporaryFile() as temp_file:
            temp_file_fd = temp_file.fileno()
            temp_file_name = temp_file.name

            # Temporary files are CLOEXEC by default
            fcntl.fcntl(temp_file_fd,
                        fcntl.F_SETFD,
                        fcntl.fcntl(temp_file_fd, fcntl.F_GETFD) &
                        ~fcntl.FD_CLOEXEC)

            # You can write with pass_fds
            p = PtyProcess.spawn(['bash',
                                  '-c',
                                  'printf hello >&{}'.format(temp_file_fd)],
                                 echo=True,
                                 pass_fds=(temp_file_fd,))
            p.wait()
            assert p.status == 0

            with open(temp_file_name, 'r') as temp_file_r:
                assert temp_file_r.read() == 'hello'

            # You can't write without pass_fds
            p = PtyProcess.spawn(['bash',
                                  '-c',
                                  'printf bye >&{}'.format(temp_file_fd)],
                                 echo=True)
            p.wait()
            assert p.status != 0

            with open(temp_file_name, 'r') as temp_file_r:
                assert temp_file_r.read() == 'hello'



import time
import unittest
from ptyprocess.ptyprocess import _is_solaris
from ptyprocess import PtyProcess

class PtyEchoTestCase(unittest.TestCase):

    def _read_until_eof(self, proc):
        """Read away all output on ``proc`` until EOF."""
        while True:
            try:
                proc.read()
            except EOFError:
                return

    @unittest.skipIf(_is_solaris, "waitnoecho cannot be called on this platform.")
    def test_waitnoecho_forever(self):
        """Ensure waitnoecho() with no timeout will return when echo=False."""
        cat = PtyProcess.spawn(['cat'], echo=False)
        assert cat.waitnoecho() == True
        assert cat.echo == False
        assert cat.getecho() == False
        cat.sendeof()
        self._read_until_eof(cat)
        assert cat.wait() == 0

    @unittest.skipIf(_is_solaris, "waitnoecho cannot be called on this platform.")
    def test_waitnoecho_timeout(self):
        """Ensure waitnoecho() with timeout will return when using stty to unset echo."""
        cat = PtyProcess.spawn(['cat'], echo=True)
        assert cat.waitnoecho(timeout=1) == False
        assert cat.echo == True
        assert cat.getecho() == True
        cat.sendeof()
        self._read_until_eof(cat)
        assert cat.wait() == 0
